import 'dart:core';
import 'dart:ui';

import 'package:flame/components.dart';
import 'package:flutter/painting.dart';

import 'digit_array.dart';
import 'engine/engine_batch.dart';
import 'engine/texture.dart';

class DigitComponent extends PositionComponent {
  final DigitArray _digitArray = DigitArray();
  final EngineBatch _engineBatch;
  final List<TextureRegion> _regions = [];
  double scale;
  double offset;

  DigitComponent(Image image,
      {EngineBatch? engineBatch, required int tileSize, this.scale = 1, this.offset = 0})
      : _engineBatch = engineBatch ?? EngineBatch() {
    var left = 0;
    for (var i = 0; i < DigitArray.digitsCount; i++) {
      var rect = Rect.fromLTWH(left.toDouble(), 0, tileSize.toDouble(), tileSize.toDouble());
      left += tileSize;
      _regions.add(TextureRegion.fromImage(image, rect: rect));
    }
  }

  int get digitValue => _digitArray.value;

  set digitValue(int value) {
    _digitArray.value = value;
  }

  TextureRegion getTextureRegion(int index) => _regions[_digitArray.getDigitalFromIndex(index)];

  @override
  void render(Canvas canvas) {
    super.render(canvas);
    _engineBatch.beginBatch(canvas);
    var offsetX = 0.0;
    for (var i = 0; i < _digitArray.characterCount; i++) {
      var textureRegion = getTextureRegion(i);
      offsetX += textureRegion.rect.width * scale + offset;
      _engineBatch.drawTextureRegion(
          textureRegion,
          RSTransform.fromComponents(
              rotation: 0,
              scale: scale,
              anchorX: 0,
              anchorY: 0,
              translateX: position.x + offsetX,
              translateY: position.y));
    }
    _engineBatch.endBatch();
  }
}
