import 'dart:typed_data';
import 'dart:ui';

class BatchData {
  int rectCount = 0;

  final rawTransforms;
  final rawSources;

  BatchData(int max)
      : rawTransforms = Float32List(max * 4),
        rawSources = Float32List(max * 4);

  void reset() {
    rectCount = 0;
  }

  void fillFromTransform(double scos, double ssin, double tx, double ty, Rect rect) {
    final index0 = rectCount * 4;
    final index1 = index0 + 1;
    final index2 = index0 + 2;
    final index3 = index0 + 3;

    rawTransforms[index0] = scos;
    rawTransforms[index1] = ssin;
    rawTransforms[index2] = tx;
    rawTransforms[index3] = ty;

    rawSources[index0] = rect.left;
    rawSources[index1] = rect.top;
    rawSources[index2] = rect.right;
    rawSources[index3] = rect.bottom;

    rectCount++;
  }

  void fillFromRSTransform(RSTransform transform, Rect rect) {
    fillFromTransform(transform.scos, transform.ssin, transform.tx, transform.ty, rect);
  }
}
