import 'dart:math';
import 'dart:ui';

import 'batch.dart';
import 'batch_data.dart';
import 'texture.dart';

class EngineBatch extends Batch {
  static const max = 1000;
  final Paint defaultPaint;
  final BlendMode defaultBlendMode;
  final BatchData _batchData = BatchData(max);

  Canvas? _canvas;
  Image? image;

  EngineBatch({BlendMode? blendMode, Paint? paint})
      : defaultBlendMode = blendMode ?? BlendMode.src,
        defaultPaint = paint ?? Paint()
          ..filterQuality = FilterQuality.low;

  @override
  void beginBatch(Canvas canvas) {
    _canvas = canvas;
  }

  @override
  void drawTextureRegion(TextureRegion textureRegion, RSTransform transform) {
    drawImage(textureRegion.texture.image, textureRegion.rect, transform);
  }

  @override
  void drawTextureRegionFromData(
      TextureRegion textureRegion, double scos, double ssin, double tx, double ty) {
    drawImageFromData(textureRegion.texture.image, textureRegion.rect, scos, ssin, tx, ty);
  }

  @override
  void drawImage(Image image, Rect rect, RSTransform transform) {
    if (this.image != image || _batchData.rectCount >= max) {
      _flush(_canvas);
    }
    this.image = image;
    _batchData.fillFromRSTransform(transform, rect);
  }

  @override
  void drawImageFromData(Image image, Rect rect, double scos, double ssin, double tx, double ty) {
    if (this.image != image || _batchData.rectCount >= max) {
      _flush(_canvas);
    }
    this.image = image;
    _batchData.fillFromTransform(scos, ssin, tx, ty, rect);
  }

  @override
  void drawImageFromComponents(
      {required Image image,
      required Rect rect,
      required double rotation,
      required double scale,
      required double anchorX,
      required double anchorY,
      required double translateX,
      required double translateY}) {
    if (this.image != image || _batchData.rectCount >= max) {
      _flush(_canvas);
    }
    final scos = cos(rotation) * scale;
    final ssin = sin(rotation) * scale;
    final tx = translateX + -scos * anchorX + ssin * anchorY;
    final ty = translateY + -ssin * anchorX - scos * anchorY;
    this.image = image;
    _batchData.fillFromTransform(scos, ssin, tx, ty, rect);
  }

  @override
  void endBatch() {
    _flush(_canvas);
  }

  void _flush(Canvas? canvas) {
    if (image == null || canvas == null) return;
    _canvas!.drawRawAtlas(image!, _batchData.rawTransforms, _batchData.rawSources, null,
        defaultBlendMode, null, defaultPaint);
    _batchData.reset();
  }
}
