import 'dart:core';
import 'dart:ui';

import 'package:flame/components.dart';
import 'package:flame/sprite.dart';
import 'package:vector_math/vector_math_64.dart';

import 'digit_array.dart';

class FlameDigitComponent extends PositionComponent {
  final DigitArray _digitArray;
  final SpriteBatch spriteBatch;
  final BlendMode? blendMode;
  final Rect? cullRect;
  final Paint? paint;
  final List<Rect> sources;
  double scale;
  double offset;

  FlameDigitComponent(this.sources, this.spriteBatch,
      {this.blendMode, this.cullRect, this.paint, this.scale = 1, this.offset = 0})
      : _digitArray = DigitArray();

  Rect _getRect(int index) => sources[index];

  int get digitValue => _digitArray.value;

  set digitValue(int value) {
    _digitArray.value = value;
  }

  @override
  void update(double dt) {
    super.update(dt);
    if (_digitArray.valueChanged) {
      spriteBatch.clear();
      var offsetX = 0.0;
      for (var i = 0; i < _digitArray.characterCount; i++) {
        var rect = _getRect(_digitArray.getDigitalFromIndex(i));
        offsetX += rect.width * scale + offset;
        spriteBatch.add(
          source: rect,
          scale: scale,
          offset: Vector2(offsetX, 0),
        );
      }
      _digitArray.valueChanged = false;
    }
  }

  @override
  void render(Canvas canvas) {
    super.render(canvas);
    spriteBatch.render(
      canvas,
      blendMode: blendMode,
      cullRect: cullRect,
      paint: paint,
    );
  }
}
